"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = __importStar(require("fs"));
class FileCacheEntry {
}
class FileCache {
    constructor(maxSize) {
        this.maxSize = maxSize;
        this.size = 0;
        this.entries = {};
        this.accessCounter = 0;
        if (maxSize < 1) {
            throw new Error('FileCache must have a max size larger than 0');
        }
    }
    getFileContents(filePath) {
        let entry = this.entries[filePath] || this.insertNewEntry(filePath);
        entry.lastAccess = this.accessCounter++;
        return entry.contents;
    }
    insertNewEntry(filePath) {
        if (this.size === this.maxSize) {
            this.evictOldest();
        }
        const newEntry = {
            lastAccess: -1,
            contents: fs.readFileSync(filePath, 'utf8').split('\n'),
        };
        this.entries[filePath] = newEntry;
        return newEntry;
    }
    evictOldest() {
        let oldestEntryFilePath;
        let oldestEntry;
        for (let filePath in this.entries) {
            const entry = this.entries[filePath];
            if (!oldestEntryFilePath || entry.lastAccess < oldestEntry.lastAccess) {
                oldestEntry = entry;
            }
        }
        delete this.entries[oldestEntryFilePath];
    }
}
exports.FileCache = FileCache;
