"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
class FileSnipper {
    constructor(sourceRoot, fileCache) {
        this.sourceRoot = sourceRoot;
        this.fileCache = fileCache;
        this.snippetRadius = 32;
    }
    getSnippet(sourceFile, lineIndex, columnIndex) {
        const filePath = path.isAbsolute(sourceFile) ? sourceFile : path.resolve(this.sourceRoot, sourceFile);
        const sourceFileContents = this.fileCache.getFileContents(filePath);
        const line = sourceFileContents[lineIndex];
        const snippetStartIndex = Math.max(columnIndex - this.snippetRadius, 0);
        const snippetEndIndex = columnIndex + this.snippetRadius;
        const prefixEllipsis = snippetStartIndex > 0 ? '...' : '';
        const suffixEllipsis = snippetEndIndex < line.length - 1 ? '...' : '';
        const snippet = prefixEllipsis + line.slice(snippetStartIndex, snippetEndIndex + 1) + suffixEllipsis;
        let columnMarker = prefixEllipsis + line.slice(snippetStartIndex, columnIndex);
        columnMarker = columnMarker.replace(/./g, ' ') + '^';
        return snippet + '\n' + columnMarker;
    }
}
exports.FileSnipper = FileSnipper;
