"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const source_map_1 = require("@lewistg/source-map");
const viewer_script_1 = require("./viewer-script");
const array_1 = require("./array");
function renderAsAnnotatedHtml(generatedCode, sourceMap, fileSnipper) {
    const mappings = source_map_1.getMappings(sourceMap);
    const mappingIndex = new Map(mappings.map((m, index) => [m, index]));
    const mappingsByLine = array_1.groupBy(mappings, (m) => m.generatedLineIndex);
    const mappingSnippets = getMappingSnippets(mappings, fileSnipper);
    const codeSpans = generatedCode.map((line, lineIndex) => {
        const lineMappings = mappingsByLine.get(lineIndex) || [];
        if (lineMappings.length == 0) {
            return [getSegmentHtml(line)];
        }
        let leadingWhiteSpace = '';
        if (lineMappings.length > 0 && lineMappings[0].generatedColumnIndex != 0) {
            leadingWhiteSpace = line.slice(0, lineMappings[0].generatedColumnIndex);
        }
        const segments = lineMappings.map((segment, index) => {
            const segmentStart = segment.generatedColumnIndex;
            const nextMapping = lineMappings[index + 1];
            const segmentEnd = (nextMapping && nextMapping.generatedColumnIndex) || line.length;
            return getSegmentHtml(line.slice(segmentStart, segmentEnd), mappingIndex.get(segment));
        });
        return leadingWhiteSpace + segments.join('');
    });
    return renderTemplate(codeSpans.join('\n'), mappings, mappingSnippets);
}
exports.renderAsAnnotatedHtml = renderAsAnnotatedHtml;
function getSegmentHtml(sourceSnippet, mappingIndex) {
    if (mappingIndex !== undefined) {
        // mi = mapping index
        return `<span data-mi="${mappingIndex}">${sourceSnippet}</span>`;
    }
    else {
        return `<span>${sourceSnippet}</span>`;
    }
}
function getMappingSnippets(mappings, fileSnipper) {
    return mappings.map(m => {
        if (m.originalLineIndex !== undefined && m.originalColumnIndex !== undefined && m.source !== undefined) {
            return fileSnipper.getSnippet(m.source, m.originalLineIndex, m.originalColumnIndex);
        }
    });
}
function renderTemplate(codeSpans, mappings, mappingSnippets) {
    const serializedMappings = mappings.map(m => source_map_1.serializeMapping(m));
    return `<!DOCTYPE html>
<html>
<head></head>
<body>
    <pre id="segments">${codeSpans}</pre>
    <script>
        var mappings = ${JSON.stringify(serializedMappings)};
        var mappingSnippets = ${JSON.stringify(mappingSnippets)};
    </script>
    <script>${viewer_script_1.viewerScript}</script>
</body>
</html>`;
}
